<?php
namespace Ak\Db\Adapter;

use Ak\Db\Exception;
use Interop\Container\ContainerInterface;
use Laminas\Db\Adapter\AdapterInterface;
use Laminas\ServiceManager\Factory\FactoryInterface;

/**
 * Vraća konfiguraciju za dati adapter.
 *
 * @package Ak\Db\Adapter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
final class AdapterConfigFactory implements FactoryInterface
{
    /**
     * Create an object
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return array
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $config = $container->get('config')['db'] ?? [];

        if ($requestedName !== 'db' && $requestedName !== AdapterInterface::class) {
            if (!isset($config['adapters'][$requestedName])) {
                throw new Exception\ConfigurationException(sprintf(
                    "Konfiguracija za db adapter '%s' ne postoji.",
                    $requestedName
                ));
            }

            $config = $config['adapters'][$requestedName];
        } else if (isset($config['default_adapter'])) {
            if (!isset($config['adapters'][$config['default_adapter']])) {
                throw new Exception\ConfigurationException(sprintf(
                    "Zadani db adapter '%s' ne postoji.",
                    $config['default_adapter']
                ));
            }

            $config = $config['adapters'][$config['default_adapter']];
        }

        if (!isset($config['driver'])) {
            throw new Exception\ConfigurationException("Konfiguracija db adaptera nema ključ 'driver'.");
        }

        if (!$container->has($config['driver'])) {
            throw new Exception\ConfigurationException(sprintf(
                "Konfiguracija db adaptera definira nepostojeći driver '%s'.",
                $config['driver']
            ));
        }

        return $config;
    }
}
