<?php
namespace Ak\Db\Adapter\Platform;

use Interop\Container\ContainerInterface;
use Laminas\Db\Adapter\Platform as LaminasPlatform;
use Laminas\Db\Adapter\Platform\PlatformInterface;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\AbstractFactoryInterface;

/**
 * Stvara novu instancu DB platforme.
 *
 * @package Ak\Db\Adapter\Platform
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
class PlatformAbstractFactory implements AbstractFactoryInterface
{
    /**
     * @param string $requestedName
     * @return string
     */
    private function getPlatformName(string $requestedName): string
    {
        return strtolower(preg_replace('/^db.platform./', '', $requestedName));
    }

    /**
     * Da li možemo stvoriti platformu s datim imenom.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     *
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName)
    {
        $platformName = $this->getPlatformName($requestedName);
        return in_array($platformName, ['mysql', 'sqlsrv', 'sqlite']);
    }

    /**
     * Stvara platformu.
     *
     * @param  ContainerInterface $container
     * @param  string $requestedName
     * @param  null|array $options
     *
     * @return PlatformInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null)
    {
        $platformName = $this->getPlatformName($requestedName);

        switch ($platformName) {
            case 'mysql':
                return new LaminasPlatform\Mysql();
            case 'sqlsrv':
                return new SqlServer();
            case 'sqlite':
                return new LaminasPlatform\Sqlite();
        }

        throw new ServiceNotCreatedException(sprintf("Ne znam napraviti platformu '%s'.", $platformName));
    }
}
