<?php
namespace AkTest\Db\Adapter;

use Ak\Db\Adapter\AdapterAbstractFactory;
use Ak\Db\Adapter\Driver\Dbal\Dbal;
use Ak\Db\Adapter\Driver\Dbal\DbalSqlite;
use Ak\Db\Adapter\Platform\SqlServer;
use Ak\Db\ConfigProvider;
use Ak\Test\AbstractTestCase;
use Laminas\Db\Adapter\AdapterInterface;
use Laminas\Db\Adapter\Platform\PlatformInterface;

/**
 * Class AdapterAbstractFactoryTest
 *
 * @package AkTest\Db\Adapter
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
class AdapterAbstractFactoryTest extends AbstractTestCase
{
    public $configProviders = [
        ConfigProvider::class,
    ];

    public function testKnowsHowToCreateAdapters()
    {
        $container = $this->getContainer([
            'db' => [
                'driver' => 'db.driver.mysql',

                'adapters' => [
                    'db.other' => [],
                ],
            ],
        ]);

        $factory = new AdapterAbstractFactory();

        $this->assertTrue($factory->canCreate($container, 'db'));
        $this->assertTrue($factory->canCreate($container, 'db.other'));
    }

    public function testDoesntKnowHowToCreateInvalidAdapters()
    {
        $container = $this->getContainer();
        $factory = new AdapterAbstractFactory();

        $this->assertFalse($factory->canCreate($container, 'db'));
        $this->assertFalse($factory->canCreate($container, 'db.any'));
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testCanCreateDefaultAdapter()
    {
        $container = $this->getContainer([
            'db' => [
                'driver' => 'db.driver.sqlite',
                'database' => ':memory:',
            ],
        ]);

        $factory = new AdapterAbstractFactory();
        $adapter = $factory($container, 'db');

        $this->assertInstanceOf(AdapterInterface::class, $adapter);
        $this->assertInstanceOf(DbalSqlite::class, $adapter->getDriver());
        $this->assertInstanceOf(PlatformInterface::class, $adapter->getPlatform());
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testCanCreateOtherAdapters()
    {
        $container = $this->getContainer([
            'db' => [
                'adapters' => [
                    'db.other' => [
                        'driver' => 'db.driver.sqlite',
                        'database' => ':memory:',
                    ],
                ],
            ],
        ]);

        $factory = new AdapterAbstractFactory();
        $adapter = $factory($container, 'db.other');

        $this->assertInstanceOf(AdapterInterface::class, $adapter);
        $this->assertInstanceOf(DbalSqlite::class, $adapter->getDriver());
        $this->assertInstanceOf(PlatformInterface::class, $adapter->getPlatform());
    }
}
