<?php
namespace AkTest\Db\Adapter\Driver\Dbal;

use Ak\Db\Adapter\Driver\Dbal\Connection;
use Ak\Db\Adapter\Driver\Dbal\DbalSqlite;
use Ak\Db\Adapter\Driver\DriverInterface;
use Ak\Test\AbstractTestCase;
use AkTest\Db\TestConfigProvider;
use Doctrine\DBAL\Schema\Schema;
use Laminas\Db\Adapter\Driver\ResultInterface;
use Laminas\Db\Adapter\Driver\StatementInterface;

/**
 * Class DbalSqliteTest
 *
 * @package AkTest\Db\Adapter\Driver\Dbal
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/db
 */
class DbalSqliteTest extends AbstractTestCase
{
    public $configProviders = [
        TestConfigProvider::class,
    ];

    /**
     * @param Schema $schema
     * @throws \Doctrine\DBAL\Schema\SchemaException
     */
    public function createSchema(Schema $schema): void
    {
        if (!$schema->hasTable('test_table')) {
            $table = $schema->createTable('test_table');
            $table->addColumn('id', 'integer', ['autoincrement' => true]);
            $table->addColumn('name', 'string');
            $table->setPrimaryKey(['id']);
        }
    }

    protected function getDataSet()
    {
        return new ArrayDataSet([
            'test_table' => [
                ['id' => 1, 'name' => 'One'],
                ['id' => 2, 'name' => 'Two'],
            ],
        ]);
    }

    public function testCorrectDriverIsLoaded()
    {
        $conn = $this->getDbalConnection();
        $driver = $conn->getDriver();

        $this->assertInstanceOf(DriverInterface::class, $driver);
    }

    public function testCreateStatementReturnsALaminasStatement()
    {
        $conn = $this->getDbalConnection();
        /** @var DbalSqlite $driver */
        $driver = $conn->getDriver();

        $stmt = $driver->createStatement('SELECT * FROM test_table');
        $this->assertInstanceOf(StatementInterface::class, $stmt);
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testCreateResultReturnsALaminasResultSet()
    {
        $conn = $this->getDbalConnection();
        /** @var DbalSqlite $driver */
        $driver = $conn->getDriver();
        /** @var Connection $connection */
        $connection = $driver->getConnection();
        $pdo = $connection->getResource();
        $stmt = $pdo->prepare('SELECT * FROM test_table');
        $stmt->execute();

        $result = $driver->createResult($stmt);
        $this->assertInstanceOf(ResultInterface::class, $result);
    }
}
