<?php
namespace AkTest\Db;

use Ak\Db\TableGateway\TableGatewayInterface;
use Ak\Test\AbstractTestCase;
use Laminas\Db\Adapter\AdapterInterface;
use Laminas\Db\Adapter\Driver\DriverInterface;
use Laminas\Db\Adapter\Platform\PlatformInterface;

/**
 * Class AdapterTest
 *
 * @package AkTest\Db
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/wp-core
 */
class ContainerTest extends AbstractTestCase
{
    public $configProviders = [
        TestConfigProvider::class,
    ];

    public function testCanCreatePlatforms()
    {
        $container = $this->getContainer();

        $this->assertContainerHasService($container,'db.platform.mysql', PlatformInterface::class);
        $this->assertContainerHasService($container,'db.platform.sqlite', PlatformInterface::class);
    }

    public function testCanCreateDrivers()
    {
        $container = $this->getContainer();

        $this->assertContainerHasService($container, 'db.driver.mysql', DriverInterface::class);
        $this->assertContainerHasService($container, 'db.driver.sqlite', DriverInterface::class);
    }

    public function testCanCreateDefaultAdapter()
    {
        $container = $this->getContainer([
            'db' => [
                'driver' => 'db.driver.sqlite',
                'database' => ':memory:',
            ],
        ]);

        $this->assertContainerHasService($container, ['db', AdapterInterface::class], AdapterInterface::class);
    }

    public function testCanCreateOtherAdapters()
    {
        $container = $this->getContainer([
            'db' => [
                'adapters' => [
                    'db.other' => [
                        'driver' => 'db.driver.sqlite',
                        'database' => ':memory:',
                    ],
                ],
            ],
        ]);

        $this->assertContainerHasService($container, 'db.other', AdapterInterface::class);
    }

    public function testCanCreateTableGateways()
    {
        $container = $this->getContainer([
            'db' => [
                'tables' => [
                    'db.table.some_table' => [
                        'name' => 'some_table',
                    ],
                ],
            ],
        ]);

        $this->assertContainerHasService($container, 'db.table.some_table', TableGatewayInterface::class);
    }
}
